import openApi from './openApi';
import urlUtil from './urlUtil';

export default {
  /**
   * web端模拟原生网络请求，加上签名
   * @param  {Object} options 请求选项
   * @param  {string} options.url 请求url
   * @param  {string} options.method 请求方式(例如GET)
   * @param  {string} options.headers 请求头
   * @param  {string} options.type 响应类型, json,text 或是 jsonp {在原生实现中其实与 json 相同)
   * @param  {string} [options.body] 请求体(JSON转string)
   * 
   * GET请求不支持body方式传递参数，请使用 url 传参。
   * 
   * @callback callback 响应结果回调
   * @param {number} status 返回的状态码
   * @param {boolean} ok 如果状态码在 200~299 之间就为真。
   * @param {string} statusText 状态描述文本
   * @param {Object | string} data 返回的数据，如果请求类型是 json 和 jsonp，则它就是一个 object ，如果不是，则它就是一个 string。
   * @param {Object} headers 响应头
   * 
   * @returns void
   */

  request(options, callback) {
    let parameters = null;
    if (options.body) {
      parameters = JSON.parse(options.body);
    }

    
    // 生成 AppKey,Secret
    openApi.setAppKeyAndSecret('gbss-bupm', 'rfGd23Yhjd92JkpWe');
    
    const urlSign = openApi.sign(options.url, parameters, options.headers);
    options.url = urlSign;

    const stream = weex.requireModule('stream');
    stream.fetch(options, callback);
  },
};
