/*eslint-disable*/
import md5 from 'md5';
import urlUtil from './urlUtil';


const Base64 = require('js-base64').Base64;

const xRequestKey = 'X-Requested-With';
const xRequestValue = 'OpenAPIRequest';
const contentTypeKey = 'Content-Type';
const contentTypeValue = 'application/json';

let accessToken = window.localStorage.getItem('accessToken') || null;
let appKey = null;
let appSecret = null;
let anonymousToken = null;

function getAnonymousToken() {
  if (anonymousToken) {
    return anonymousToken;
  }
  let token = `${Math.ceil(Math.random() * 100000000)}##ANONYMOUS`;
  token = Base64.encode(token);
  return token;
}

function md5Signature(parameters, secret) {
  let result = secret;

  for (const key of Object.keys(parameters).sort()) {
    result += key + parameters[key];
  };
  result += secret;
  result = md5(result).toUpperCase();
  return result;
}

export default {
  setAccessToken(token) {
    accessToken = token;
  },
  getAccessToken() {
    console.log('当前账号的token是：', accessToken)
  },
  setAppKeyAndSecret(key, secret) {
    appKey = key;
    appSecret = secret;
  },

  setAnonymousToken(token) {
    anonymousToken = token;
  },

  /**
   * 模拟原生签名
   * @param  {string} url 请求url(若是get请求方式，url附带请求参数)
   * @param  {Object} parameters 非get请求方式带的请求体
   * @param  {Object} headers 请求头
   * 
   * @returns {string} 拼接S，再将S进行md5加密后转大写返回
   *                   格式S:
   *                   S = ’${app_secret}\
   * 
   *                        ${access-token}${itsValue}\
   *                        ${timestamp}${itsValue}\
   *                        ${app_key}${itsValue}\
   *                        ${No.1_papamOf_BodyOrUrlQuery}${itsValue}\
   *                                    .
   *                                    .
   *                        ${No.n_papamOf_BodyOrUrlQuery}${itsValue}\
   * 
   *                        ${app_secret}’;
   * 
   *                   注:（app_secret包围的键值对按键名升序排列）
   */
  sign(url, parameters, headers) {
    let isOpenApi = false;

    let isJson = false;
    if (headers != null) {
      for (const key in headers) {
        if (key === xRequestKey && headers[key] === xRequestValue) {
          isOpenApi = true;
        }
        if (key === contentTypeKey && headers[key].includes(contentTypeValue)) {
          isJson = true;
        }
      }
    }

    if (!isOpenApi) {
      return url;
    }

    const querySting = urlUtil.getQuery(url);
    let queryCombined = querySting;
    let bodySting = null;
    if (parameters != null) {
      if (isJson) {
        bodySting = `openApiBodyString=${encodeURIComponent(JSON.stringify(parameters))}`;
      } else {
        bodySting = urlUtil.parametersToQuery(parameters);
      }
      if (queryCombined != null) {
        queryCombined += `&${bodySting}`;
      } else {
        queryCombined = bodySting;
      }
    }

    let parametersSigned = {};
    if (queryCombined != null) {
      parametersSigned = urlUtil.queryToParameters(queryCombined);
    }
    const token = accessToken != null ? accessToken : getAnonymousToken();
    const timestamp = new Date().getTime();

    let urlSigned = url;
    if (!urlSigned.includes('?')) {
      urlSigned += '?';
    } else if (querySting != null) {
      urlSigned += '&';
    }
    urlSigned += `access_token=${encodeURIComponent(token)}`;
    urlSigned += `&timestamp=${timestamp}`;
    urlSigned += `&app_key=${encodeURIComponent(appKey)}`;

    parametersSigned.access_token = token;
    parametersSigned.timestamp = timestamp;
    parametersSigned.app_key = appKey;
    const sign = md5Signature(parametersSigned, appSecret);
    urlSigned += `&sign=${encodeURIComponent(sign)}`;
    return urlSigned;
  },
};
